@@ -16,9 +16,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -102,10 +100,8 @@ enum {
 G_DEFINE_TYPE (WnckActionMenu, wnck_action_menu, GTK_TYPE_MENU);
 #define WNCK_ACTION_MENU_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), WNCK_TYPE_ACTION_MENU, WnckActionMenuPrivate))
 
-static void wnck_action_menu_finalize (GObject *object);
+static void wnck_action_menu_dispose (GObject *object);
 
-static void object_weak_notify (gpointer data,
-                                GObject *obj);
 static void window_weak_notify (gpointer data,
                                 GObject *window);
 
@@ -116,23 +112,10 @@ static void
 window_weak_notify (gpointer data,
                     GObject *window)
 {
-  g_object_weak_unref (G_OBJECT (data),
-                       object_weak_notify,
-                       window);
-
+  WNCK_ACTION_MENU(data)->priv->window = NULL;
   gtk_widget_destroy (GTK_WIDGET (data));
 }
 
-
-static void
-object_weak_notify (gpointer data,
-                    GObject *obj)
-{
-  g_object_weak_unref (G_OBJECT (data),
-                       window_weak_notify,
-                       obj);
-}
-
 static WnckActionMenu*
 get_action_menu (GtkWidget *widget)
 {
@@ -353,38 +336,10 @@ set_item_text (GtkWidget  *mi,
   GtkLabel *label;
 
   label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (mi)));
-  gtk_label_set_text (label, text);
+  gtk_label_set_text_with_mnemonic (label, text);
   gtk_label_set_use_underline (label, TRUE);
 }
 
-static void
-set_item_stock (GtkWidget  *mi,
-                const char *stock_id)
-{
-  GtkWidget *image;
-
-  image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (mi));
-
-  if (stock_id == NULL)
-    {
-      if (image != NULL)
-        gtk_widget_destroy (image);
-      return;
-    }
-
-  if (image == NULL)
-    {
-      image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU);
-      gtk_widget_show (image);
-      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
-    }
-  else
-    {
-      gtk_image_set_from_stock (GTK_IMAGE (image), stock_id,
-                                GTK_ICON_SIZE_MENU);
-    }
-}
-
 static gboolean
 update_menu_state (WnckActionMenu *menu)
 {
@@ -411,14 +366,12 @@ update_menu_state (WnckActionMenu *menu)
   if (wnck_window_is_minimized (priv->window))
     {
       set_item_text (priv->minimize_item, _("Unmi_nimize"));
-      set_item_stock (priv->minimize_item, NULL);
       gtk_widget_set_sensitive (priv->minimize_item,
                                 (actions & WNCK_WINDOW_ACTION_UNMINIMIZE) != 0);
     }
   else
     {
       set_item_text (priv->minimize_item, _("Mi_nimize"));
-      set_item_stock (priv->minimize_item, WNCK_STOCK_MINIMIZE);
       gtk_widget_set_sensitive (priv->minimize_item,
                                 (actions & WNCK_WINDOW_ACTION_MINIMIZE) != 0);
     }
@@ -426,14 +379,12 @@ update_menu_state (WnckActionMenu *menu)
   if (wnck_window_is_maximized (priv->window))
     {
       set_item_text (priv->maximize_item, _("Unma_ximize"));
-      set_item_stock (priv->maximize_item, NULL);
       gtk_widget_set_sensitive (priv->maximize_item,
                                 (actions & WNCK_WINDOW_ACTION_UNMAXIMIZE) != 0);
     }
   else
     {
       set_item_text (priv->maximize_item, _("Ma_ximize"));
-      set_item_stock (priv->maximize_item, WNCK_STOCK_MAXIMIZE);
       gtk_widget_set_sensitive (priv->maximize_item,
                                 (actions & WNCK_WINDOW_ACTION_MAXIMIZE) != 0);
     }
@@ -710,7 +661,7 @@ make_menu_item (WindowAction action)
 {
   GtkWidget *mi;
 
-  mi = gtk_image_menu_item_new_with_label ("");
+  mi = gtk_menu_item_new_with_label ("");
 
   g_signal_connect (G_OBJECT (mi), "activate",
                     G_CALLBACK (item_activated_callback),
@@ -835,7 +786,6 @@ refill_submenu_workspace (WnckActionMenu *menu)
 
       gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
       set_item_text (item, name);
-      set_item_stock (item, NULL);
 
       g_free (name);
     }
@@ -914,7 +864,6 @@ refill_submenu_viewport (WnckActionMenu *menu)
 
           gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
           set_item_text (item, label);
-          set_item_stock (item, NULL);
 
           g_free (label);
         }
@@ -1024,7 +973,6 @@ wnck_action_menu_constructor (GType                  type,
     }
 
   g_object_weak_ref (G_OBJECT (priv->window), window_weak_notify, menu);
-  g_object_weak_ref (G_OBJECT (menu), object_weak_notify, priv->window);
 
   priv->minimize_item = make_menu_item (MINIMIZE);
 
@@ -1041,14 +989,12 @@ wnck_action_menu_constructor (GType                  type,
                          priv->move_item);
 
   set_item_text (priv->move_item, _("_Move"));
-  set_item_stock (priv->move_item, NULL);
 
   priv->resize_item = make_menu_item (RESIZE);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          priv->resize_item);
 
   set_item_text (priv->resize_item, _("_Resize"));
-  set_item_stock (priv->move_item, NULL);
 
   priv->workspace_separator = separator = gtk_separator_menu_item_new ();
   gtk_widget_show (separator);
@@ -1077,25 +1023,21 @@ wnck_action_menu_constructor (GType                  type,
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          priv->left_item);
   set_item_text (priv->left_item, _("Move to Workspace _Left"));
-  set_item_stock (priv->left_item, NULL);
 
   priv->right_item = make_menu_item (RIGHT);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          priv->right_item);
   set_item_text (priv->right_item, _("Move to Workspace R_ight"));
-  set_item_stock (priv->right_item, NULL);
 
   priv->up_item = make_menu_item (UP);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          priv->up_item);
   set_item_text (priv->up_item, _("Move to Workspace _Up"));
-  set_item_stock (priv->up_item, NULL);
 
   priv->down_item = make_menu_item (DOWN);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          priv->down_item);
   set_item_text (priv->down_item, _("Move to Workspace _Down"));
-  set_item_stock (priv->down_item, NULL);
 
   priv->workspace_item = gtk_menu_item_new_with_mnemonic (_("Move to Another _Workspace"));
   gtk_widget_show (priv->workspace_item);
@@ -1118,7 +1060,6 @@ wnck_action_menu_constructor (GType                  type,
                          priv->close_item);
 
   set_item_text (priv->close_item, _("_Close"));
-  set_item_stock (priv->close_item, WNCK_STOCK_DELETE);
 
   g_signal_connect_object (G_OBJECT (priv->window),
                            "state_changed",
@@ -1168,14 +1109,12 @@ wnck_action_menu_class_init (WnckActionMenuClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  _wnck_stock_icons_init ();
-
   g_type_class_add_private (klass, sizeof (WnckActionMenuPrivate));
 
   object_class->constructor = wnck_action_menu_constructor;
   object_class->get_property = wnck_action_menu_get_property;
   object_class->set_property = wnck_action_menu_set_property;
-  object_class->finalize = wnck_action_menu_finalize;
+  object_class->dispose = wnck_action_menu_dispose;
 
   g_object_class_install_property (object_class,
                                    PROP_WINDOW,
@@ -1186,17 +1125,30 @@ wnck_action_menu_class_init (WnckActionMenuClass *klass)
 }
 
 static void
-wnck_action_menu_finalize (GObject *object)
+wnck_action_menu_dispose (GObject *object)
 {
   WnckActionMenu *menu;
 
   menu = WNCK_ACTION_MENU (object);
 
   if (menu->priv->idle_handler)
-    g_source_remove (menu->priv->idle_handler);
-  menu->priv->idle_handler = 0;
+    {
+      g_source_remove (menu->priv->idle_handler);
+      menu->priv->idle_handler = 0;
+    }
 
-  G_OBJECT_CLASS (wnck_action_menu_parent_class)->finalize (object);
+  if (WNCK_IS_WINDOW (menu->priv->window))
+    {
+      g_object_weak_unref (G_OBJECT (menu->priv->window), window_weak_notify, menu);
+      g_signal_handlers_disconnect_by_data (menu->priv->window, menu);
+
+      WnckScreen *screen = wnck_window_get_screen (menu->priv->window);
+      g_signal_handlers_disconnect_by_data (screen, menu);
+
+      menu->priv->window = NULL;
+    }
+
+  G_OBJECT_CLASS (wnck_action_menu_parent_class)->dispose (object);
 }
 
 /**
