@@ -1,6 +1,6 @@
 /* vim: set sw=2 et: */
 /*
- * Copyright (C) 2007 Vincent Untz 
+ * Copyright (C) 2007 Vincent Untz
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors:
  *	Vincent Untz <vuntz@gnome.org>
@@ -38,6 +36,7 @@
 #include <string.h>
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
+#include <X11/extensions/XInput2.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -45,7 +44,6 @@
 #include <glib/gi18n.h>
 
 #include <libwnck/libwnck.h>
-#include <libwnck/class-group.h>
 
 enum {
   INVALID_MODE,
@@ -296,7 +294,7 @@ static GOptionEntry space_entries[] = {
 static void clean_up (void);
 
 /* this part is mostly stolen from xutils.c */
-typedef struct 
+typedef struct
 {
   Window window;
   Atom timestamp_prop_atom;
@@ -1348,10 +1346,13 @@ print_class_group (WnckClassGroup *class_group)
 
   windows = wnck_class_group_get_windows (class_group);
 
-  /* Translators: Resource class is the name to identify a class. */
-  g_print (_("Resource Class: %s\n"),
-           wnck_class_group_get_res_class (class_group));
-  g_print (_("Group Name: %s\n"), wnck_class_group_get_name (class_group));
+  /* Translators: A class is like a "family". E.g., all gvim windows are of the
+   * same class. */
+  g_print (_("Class Group ID: %s\n"),
+           wnck_class_group_get_id (class_group));
+  /* Translators: A class is like a "family". E.g., all gvim windows are of the
+   * same class. */
+  g_print (_("Class Group Name: %s\n"), wnck_class_group_get_name (class_group));
 
   /* TODO: missing API */
 #if 0
@@ -1499,14 +1500,24 @@ print_window (WnckWindow *window)
   g_print (_("Geometry (x, y, width, height): %d, %d, %d, %d\n"), x, y, w, h);
 
   class_group = wnck_window_get_class_group (window);
-  if (strcmp (wnck_class_group_get_res_class (class_group), ""))
-    buf = wnck_class_group_get_res_class (class_group);
+  if (strcmp (wnck_class_group_get_id (class_group), ""))
+    buf = wnck_class_group_get_id (class_group);
   else
     /* Translators: 'unset' in the sense of "something has not been set". */
     buf = _("<unset>");
   /* Translators: A class is like a "family". E.g., all gvim windows are of the
    * same class. */
-  g_print (_("Class Group: %s\n"), buf);
+  g_print (_("Class Group ID: %s\n"), buf);
+
+  if (g_strcmp0 (wnck_window_get_class_instance_name (window), ""))
+    buf = wnck_window_get_class_instance_name (window);
+  else
+    /* Translators: 'unset' in the sense of "something has not been set". */
+    buf = _("<unset>");
+  /* Translators: A class is like a "family". E.g., all gvim windows are of the
+   * same class. The class instance is a way to differentiate windows belonging
+   * to the same class group. */
+  g_print (_("Class Instance: %s\n"), buf);
 
   g_print (_("XID: %lu\n"), wnck_window_get_xid (window));
 
@@ -1525,6 +1536,13 @@ print_window (WnckWindow *window)
     buf = _("<unset>");
   g_print (_("Session ID: %s\n"), buf);
 
+  if (wnck_window_get_role (window))
+    buf = wnck_window_get_role (window);
+  else
+    /* Translators: 'unset' in the sense of "something has not been set". */
+    buf = _("<unset>");
+  g_print (_("Role: %s\n"), buf);
+
   if (wnck_window_get_group_leader (window) != wnck_window_get_xid (window))
     /* Translators: A group leader is the window that is the "owner" of a group
      * of windows, ie: if you have multiple windows in one application, one
@@ -1725,13 +1743,13 @@ find_managed_window (Display *display,
   return retval;
 }
 
-static void 
-handle_button_press_event (XKeyEvent *event)
+static void
+handle_button_press_event (Display *dpy, XIDeviceEvent *event)
 {
-  if (event->subwindow == None)
+  if (event->child == None)
     return;
 
-  got_from_user = find_managed_window (event->display, event->subwindow);
+  got_from_user = find_managed_window (dpy, event->child);
 }
 
 static GdkFilterReturn
@@ -1740,22 +1758,32 @@ target_filter (GdkXEvent *gdk_xevent,
                gpointer   data)
 {
   XEvent *xevent = (XEvent *) gdk_xevent;
+  XGenericEventCookie *cookie = &xevent->xcookie;
 
-  switch (xevent->type)
+  /* Use XI2 to read the event data */
+  if (cookie->type == GenericEvent)
     {
-      case ButtonPress:
-        handle_button_press_event (&xevent->xkey);
-        clean_up ();
-        return GDK_FILTER_REMOVE;
-      case KeyPress:
-        if (xevent->xkey.keycode == XKeysymToKeycode (xevent->xany.display, XK_Escape))
-          {
+      XIDeviceEvent *event = cookie->data;
+
+      if (!event)
+        return GDK_FILTER_CONTINUE;
+
+      switch (event->evtype)
+        {
+          case XI_ButtonPress:
+            handle_button_press_event (cookie->display, event);
             clean_up ();
             return GDK_FILTER_REMOVE;
-          }
-        break;
-      default:
-        break;
+          case XI_KeyPress:
+            if (event->detail == XKeysymToKeycode (cookie->display, XK_Escape))
+              {
+                clean_up ();
+                return GDK_FILTER_REMOVE;
+              }
+            break;
+          default:
+            break;
+        }
     }
 
   return GDK_FILTER_CONTINUE;
@@ -1764,18 +1792,25 @@ target_filter (GdkXEvent *gdk_xevent,
 static gboolean
 get_target (gpointer data)
 {
-  GdkGrabStatus  status;
-  GdkCursor     *cross;
-  GdkWindow     *root;
+  GdkDisplay       *display;
+  GdkGrabStatus     status;
+  GdkDeviceManager *dev_manager;
+  GdkDevice        *device;
+  GdkCursor        *cross;
+  GdkWindow        *root;
+  GList            *devices, *l;
 
+  display = gdk_display_get_default ();
+  dev_manager = gdk_display_get_device_manager (display);
   root = gdk_get_default_root_window ();
 
   gdk_window_add_filter (root, (GdkFilterFunc) target_filter, NULL);
 
-  cross = gdk_cursor_new (GDK_CROSS);
-  status = gdk_pointer_grab (root, FALSE, GDK_BUTTON_PRESS_MASK,
-                             NULL, cross, GDK_CURRENT_TIME);
-  gdk_cursor_unref (cross);
+  cross = gdk_cursor_new_for_display (display, GDK_CROSS);
+  device = gdk_device_manager_get_client_pointer (dev_manager);
+  status = gdk_device_grab (device, root, GDK_OWNERSHIP_WINDOW, TRUE,
+                            GDK_BUTTON_PRESS_MASK, cross, GDK_CURRENT_TIME);
+  g_object_unref (cross);
 
   if (status != GDK_GRAB_SUCCESS)
     {
@@ -1784,7 +1819,25 @@ get_target (gpointer data)
       return FALSE;
     }
 
-  status = gdk_keyboard_grab (root, FALSE, GDK_CURRENT_TIME);
+  devices = gdk_device_manager_list_devices (dev_manager, GDK_DEVICE_TYPE_MASTER);
+
+  for (l = devices; l; l = l->next)
+    {
+      device = GDK_DEVICE (l->data);
+
+      if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
+        continue;
+
+      status = gdk_device_grab (device, root, GDK_OWNERSHIP_NONE, TRUE,
+                                GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK, NULL,
+                                GDK_CURRENT_TIME);
+
+      if (status != GDK_GRAB_SUCCESS)
+        break;
+    }
+
+  g_list_free (devices);
+
   if (status != GDK_GRAB_SUCCESS)
     {
       g_warning ("Keyboard grab failed.\n");
@@ -1801,12 +1854,30 @@ static void
 clean_up (void)
 {
   GdkWindow *root;
+  GdkDeviceManager *dev_manager;
+  GdkDevice        *device;
+  GList            *devices, *l;
 
   root = gdk_get_default_root_window ();
+  dev_manager = gdk_display_get_device_manager (gdk_display_get_default ());
   gdk_window_remove_filter (root, (GdkFilterFunc) target_filter, NULL);
 
-  gdk_pointer_ungrab (GDK_CURRENT_TIME);
-  gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+  device = gdk_device_manager_get_client_pointer (dev_manager);
+  gdk_device_ungrab (device, GDK_CURRENT_TIME);
+
+  devices = gdk_device_manager_list_devices (dev_manager, GDK_DEVICE_TYPE_MASTER);
+
+  for (l = devices; l; l = l->next)
+    {
+      device = GDK_DEVICE (l->data);
+
+      if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
+        continue;
+
+      gdk_device_ungrab (device, GDK_CURRENT_TIME);
+    }
+
+  g_list_free (devices);
 
   gtk_main_quit ();
 }
@@ -1818,7 +1889,7 @@ main (int argc, char **argv)
   GOptionGroup   *group;
   GError         *error;
   WnckScreen     *screen;
-  
+
   bindtextdomain (GETTEXT_PACKAGE, WNCK_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
@@ -1924,7 +1995,7 @@ main (int argc, char **argv)
       if (!got_from_user)
         return 0;
     }
-  
+
   if (mode == SCREEN_READ_MODE)
     print_screen (screen);
   else if (mode == SCREEN_LIST_MODE)
@@ -2025,6 +2096,6 @@ main (int argc, char **argv)
         g_printerr (_("Cannot interact with window with XID %lu: "
                       "the window cannot be found\n"), xid);
     }
-  
+
   return 0;
 }
